//------------------------------------------------------------
// Barbent Servo GT Clockworks V0.52
// written by Barbent/EK
// Shows up several interesting informations in a small box
// Several mode changes are available per mouseclick
// Right-clicks e.g. set to zero
//------------------------------------------------------------


angular.module('beamng.apps')
.directive('bsgtClock', [function () {
return {
templateUrl: '/ui/modules/apps/BSGT_clock/app.html',
replace: true,
restrict: 'EA',
require: '^bngApp',
link: function (scope, element, attrs, ctrl) {
	
	var settings_file_path = "settings/BSGT_clock/settings.json";
	
	// The current overlay screen the user is on (default: null)
	scope.overlayScreen = null;
	var dt = performance.now();				// Deltazeit zwischen den Updates (frei)
	var myAltitude = 0;						// Copy fo Altitude for fileglobal use
	var myZeroAltitude = 0;					// The relative Zero (offset)
	var altiMode = 1;						// 1=Absolute, 2=Realtive
	var wghtMode = 2;						// 1=Measure Part Weights, 2=Measure Tire Pressure, 3=Load, 4=Measure vertical G-Force
	var wghtValid = true;					// Nicht erfolgreiche Refendruckmessung -> für Darstellungsfarbe
	var clockMode = 2;						// 1=Gametime, 2=Reatime, 3=Drivetime
	var driveTime = 0;						// Bewegungsdauer des Vehicles
	var driveTime_frozen = true;			// Stopuhr für die Fahrtzeit angehalten
	var speedMode = 1;						// 1=GPS 3D Speed, 2=V-Max
	var vMax = 0;							// Maximal gemessene GPS-Geschwindigkeit
	var accelTo100 = 1000;					// von 0 auf 100 (1000s entspricht "nie oder nicht getestet")
	var accelTo200 = 1000;					// von 0 auf 200 (1000s entspricht "nie oder nicht getestet")
	var accelTo300 = 1000;					// von 0 auf 200 (1000s entspricht "nie oder nicht getestet")
	var accelTo100probe = false;			// true = Messung läuft
	var accelTo200probe = false;			// true = Messung läuft
	var accelTo300probe = false;			// true = Messung läuft
	var fuelMode = 1;						// 1=Litre/kWh, 2=Percent
	var tempMode = 1;						// 1=Oiltemp, 2=Watertemp
	var enableAutoHeatOverride = true;		// System kann Temperaturmodus umschalten (wenn Überhitzt)
	var autoHeatOverrideEngaged = false;	// System hat Temperaturmodus umgeschalten (wegen Überhitzung)
	const oilOverheat = 115;				// Oil Warning Teperature 
	const waterOverheat = 115;				// Water Warning Teperature 
	const goatHeatTime = 3;					// Zeit, nach der [enableAutoHeatOverride] wieder zugelassen wird
	var goatHeatTimer = goatHeatTime;		// Timer, der [enableAutoHeatOverride] wieder zu lässt
	var oldSeconds = -1;					// Zwischenspeicher für (Deltatime) Sekundenflanke
	element.ready(function () {	
		bngApi.engineLua("jsonReadFile('" + settings_file_path + "')", function(data) {
			if(data !== undefined) {
				if(data.clockMode !== undefined) clockMode = data.clockMode;
				if(data.driveTime !== undefined) driveTime = data.driveTime;
				if(data.speedMode !== undefined) speedMode = data.speedMode;
				if(data.vMax !== undefined) vMax = data.vMax;
				if(data.accelTo100 !== undefined) accelTo100 = data.accelTo100;
				if(data.accelTo200 !== undefined) accelTo200 = data.accelTo200;
				if(data.accelTo300 !== undefined) accelTo300 = data.accelTo300;
				if(data.altiMode !== undefined) altiMode = data.altiMode;
				if(data.myZeroAltitude !== undefined) myZeroAltitude = data.myZeroAltitude;
				if(data.fuelMode !== undefined) fuelMode = data.fuelMode;
				if(data.tempMode !== undefined) tempMode = data.tempMode;
				if(data.wghtMode !== undefined) wghtMode = data.wghtMode;
			}
		});

/*		var parse;
		parse = localStorage.getItem("clockMode");
		if (parse != null) clockMode = JSON.parse(parse);
		parse = localStorage.getItem("driveTime");
		if (parse != null) driveTime = JSON.parse(parse);

		parse = localStorage.getItem("speedMode");
		if (parse != null) speedMode = JSON.parse(parse);
		parse = localStorage.getItem("vMax");
		if (parse != null) vMax = JSON.parse(parse);

		parse = localStorage.getItem("altiMode");
		if (parse != null) altiMode = JSON.parse(parse);
		parse = localStorage.getItem("myZeroAltitude");
		if (parse != null) myZeroAltitude = JSON.parse(parse);
*/
	});

	var streamsList = ['electrics', 'stats', 'wheelInfo', 'engineInfo', 'sensors']
	StreamsManager.add(streamsList)
		
//------------------------------------------------------------
// Hilfsfunktionen -------------------------------------------

	//------------------------------------------------------------
	//*** Echte Uhrzeit ermitteln
	function getRealTime(){
		//Code from BeamNG's Simple Time app.js

		var currentDate = new Date();
		var currentHour = currentDate.getHours();
		var currentMinute = currentDate.getMinutes();
		var currentSeconds = currentDate.getSeconds();		// Nun mit Sekunden...

		// Vielleicht besser so in der Art (wie bei getGameTime)
		return ("0" + currentHour).slice(-2) + ":" + ("0" + currentMinute).slice(-2)+ ":" + ("0" + currentSeconds).slice(-2);
		//return ("0" + currentHour).slice(-2) + ":" + ("0" + currentMinute).slice(-2);	// Ohne Sekunden
	}

	//------------------------------------------------------------
	//*** Ingame Time ermitteln
	var gametime = null;
	function getGameTime(){
		bngApi.engineLua('core_environment.getTimeOfDay().time', function(data) {
			//0  = 0
			//12 = 0.5
			//24 = 1
			var time_seconds = data;

			if(time_seconds >= 0.5){
				time_seconds -= 0.5;
			}
			else{
				time_seconds += 0.5;
			}

			time_seconds = Math.floor(time_seconds * 86400);
			var hours = Math.floor(time_seconds / 3600);
			var minutes = Math.floor(time_seconds % 3600 / 60);

			gametime = ("0" + hours).slice(-2) + ":" + ("0" + minutes).slice(-2);
			
			// ***** Siehe auch:
			// core_environment.setTimeOfDay({play=true})
			// um dem Play-Status der Gametime zu beeinflussen
		});
		return gametime;	
	}

	//------------------------------------------------------------
	// *** Gefahrene Zeit formatieren
	function formatDriveTime(driveT){
		var dt_hour = Math.floor(driveT / 1000 / 3600);
		var dt_min = Math.floor(driveT / 1000 / 60) - dt_hour*60;
		var dt_sec = Math.floor(driveT / 1000) - dt_min*60 - dt_hour*3600;;
		return ("0" + dt_hour).slice(-2)+ ":" +("0" + dt_min).slice(-2) + ":" + ("0" + dt_sec).slice(-2);
	}

	//------------------------------------------------------------
	//*** Gewicht des Fahrzeuges verschieden ermitteln
	function getVehicleWeight(streams, mode){
		if (mode == 1) {
			// Variante 1 (incl. Sprit und Config.-Ladung).
			// Abgefallene Fahrzeugteile werden NICHT berücksichtigt, denn sie "gehören" ja noch zum Fahrzeug.
			// Keine Messung sondern Ingame-Info der Summe aller Node-Weights
			wghtValid = true;
			return "MASS: " + Math.round(streams.stats.total_weight) + "kg";
		}
		else if (mode == 2) {
			// Variante 2 (incl. Sprit und Config.-Ladung und alles andere zusätzliche obendrauf).
			// Abgefallene Fahrzeugteile WERDEN berücksichtigt.
			// Messung über den Anpressdruck der Räder
			var totalDownForce = 0;									// Gesamt-Gewichtskraft-Sammler auf 0 setzen
			var wheelCount = -1;									// Anzahl der Räder minus 1
			for (wheelCount in streams.wheelInfo) {					// für jedes Rad der Anzahl der Räder
				totalDownForce += streams.wheelInfo[wheelCount][7];	// addiere "Downforce" (Gewichtskraft)
			}
			
			if (wheelCount == -1) {									// Objekt hat keine Räder?
				wghtValid = false;
				return "WGHT: -no sensor-";
			}
			else {
				wghtValid = true;
				return "WGHT: " + Math.round(totalDownForce / 9.81) + "kg / (" + (Number(wheelCount)*1+1) + ")";
			}
		}
		else if (mode == 3) {
			wghtValid = true;
//			return "G-FORCE: " + Math.round(streams.electrics.verticalVelocity).toFixed(1) + "Nm";
//			return "G-FORCE: " + streams.electrics.verticalVelocity;
			var gf = streams.sensors.gz / -9.81
			return "G-FORCE: " + gf.toFixed(2);
		}

		wghtValid = false;
		return "Inv. Mode";		// Kein gescheiter Modus (Ermittlungs-Variante) angegeben
	}
	
	//------------------------------------------------------------
	// *** Beschleunigung (0 auf 100,200,300)
	function formatAcceleration(value){
		var stringtemp;
		if(value < 10) {
			stringtemp = value.toFixed(2) + "s";
		}
		else if(value < 100) {
			stringtemp = value.toFixed(1) + "s";
		}
		else if(value < 1000) {
			stringtemp = value.toFixed(0) + "s";
		}
		else {
			stringtemp = "---";
		}
		return stringtemp;
	}

// ------------------------------------------------------------

	scope.$on('streamsUpdate', function (event, streams) {
		scope.$evalAsync(function () {
			// Sekunden holen zum Blinken von Anzeigen
			var currentDate = new Date();	// TODO(andere Zeitbasis für Blinkeffekt nutzen)
			
			dt = performance.now() - dt;	// Zeit, seit dem letzten Aufruf
			// Drivetime sammeln
			driveTime_frozen = true;								// Status: Uhr angehalten
			if (streams.electrics.airspeed) {							// Sensor da?
				if (streams.electrics.airspeed * 3.6 > 1) {				// V bewegt sich?
					driveTime += dt;
					// Überlauf abfangen
					if (driveTime >= 360000000) { driveTime = 0; }		//Maximum 100h
					driveTime_frozen = false;							// Status: Uhr läuft
				}
			}

	// -------- Daten anzeigen:
			
			//------------------------------------------------------------
			// *** Uhrzeiten
			if (clockMode == 1) {
				scope.clockstyle = "color:white";
				scope.clocktime = "GAME: " + getGameTime();
			}
			else if (clockMode == 2) {
				scope.clockstyle = "color:white";
				scope.clocktime = "REAL: " + getRealTime();
			}
			else if (clockMode == 3) {
				if (driveTime_frozen) {
					scope.clockstyle = "color:LightGray";
					scope.clocktime = "TRIP: " + formatDriveTime(driveTime) + " [ll]";
				}
				else {
					scope.clockstyle = "color:white";
					scope.clocktime = "TRIP: " + formatDriveTime(driveTime);
				}
			}
							
			//------------------------------------------------------------
			// *** SPEED Geschwindigkeit im Gas (Airspeed)
			// Topspeed updaten
			if (streams.electrics.airspeed > vMax) { vMax = streams.electrics.airspeed }

			//----------------------------------------------------------------------------------------------
			// Auto Acceleration STOPWATCHES
			
		//	// von 0 auf 100 Stopwatch update
			if (accelTo100probe == false) {
				if (accelTo100 == 1000 && streams.electrics.airspeed > 0.2 && speedMode == 3) { //never und speed und modus
					// Messung starten
					accelTo100 = 0;
					accelTo100probe = true;
				}
			}
			else {
				if (accelTo100 <= 1000) {					// Messung dauert noch nicht zu lange?
					// Messen
					accelTo100 = accelTo100 + (dt/1000);	//Sekunden addieren
				}	// Sonst einfach nicht weiter zählen
				
				if (streams.electrics.airspeed <= 0.2) {		// Fahrz stop?
					accelTo100probe = false;	// Messung stoppen
					accelTo100 = 1000;			// Never mit Neustart
				}
				
				if (streams.electrics.airspeed * 3.6 >= 100) {	// 100 km/h
					accelTo100probe = false;	// Messung beenden
				}
			}
		//	// von 0 auf 200 Stopwatch update
			if (accelTo200probe == false) {
				if (accelTo200 == 1000 && streams.electrics.airspeed > 0.2  && speedMode == 3) { //never und speed und modus
					// Messung starten
					accelTo200 = 0;
					accelTo200probe = true;
				}
			}
			else {
				if (accelTo200 <= 1000) {					// Messung dauert noch nicht zu lange?
					// Messen
					accelTo200 = accelTo200 + (dt/1000);	//Sekunden addieren
				}	// Sonst einfach nicht weiter zählen
				
				if (streams.electrics.airspeed <= 0.2) {		// Fahrz stop?
					accelTo200probe = false;	// Messung stoppen
					accelTo200 = 1000;			// Never mit Neustart
				}
				
				if (streams.electrics.airspeed * 3.6 >= 200) {	// 200 km/h
					accelTo200probe = false;	// Messung beenden
				}
			}
		//	// von 0 auf 300 Stopwatch update
			if (accelTo300probe == false) {
				if (accelTo300 == 1000 && streams.electrics.airspeed > 0.2  && speedMode == 3) { //never und speed und modus
					// Messung starten
					accelTo300 = 0;
					accelTo300probe = true;
				}
			}
			else {
				if (accelTo300 <= 1000) {					// Messung dauert noch nicht zu lange?
					// Messen
					accelTo300 = accelTo300 + (dt/1000);	//Sekunden addieren
				}	// Sonst einfach nicht weiter zählen
				
				if (streams.electrics.airspeed <= 0.2) {		// Fahrz stop?
					accelTo300probe = false;	// Messung stoppen
					accelTo300 = 1000;			// Never mit Neustart
				}
				
				if (streams.electrics.airspeed * 3.6 >= 300) {	// 200 km/h
					accelTo300probe = false;	// Messung beenden
				}
			}
			//----------------------------------------------------------------------------------------------
			// Display
			if(speedMode == 1) {
				scope.speedstyle = "color:white"
				if (streams.electrics.airspeed * 3.6 > 0.01 && streams.electrics.airspeed * 3.6 < 1) {// Sonderfall zwischen 0 und eins
					scope.airspeed = "V3D: slow";
				}
				else {
					scope.airspeed = "V3D: " + Math.round(streams.electrics.airspeed * 3.6) + " km/h";
				}
			}
			else if(speedMode == 2) {
				vMax * 3.6 >= 1 ? scope.speedstyle = "color:white" : scope.speedstyle = "color:LightGray";
				if (vMax * 3.6 > 0.01 && vMax * 3.6 < 1) {// Sonderfall zwischen 0 und eins
					scope.airspeed = "VMAX: slow";
				}
				else {
					scope.airspeed = "VMAX: " + Math.round(vMax * 3.6) + " km/h";
				}
			}
			else if(speedMode == 3) {
				// Output of the above geathered data
				if (accelTo100 == 1000 && accelTo200 == 1000 && accelTo300 == 1000) {
					scope.speedstyle = "color:LightGray";
				}
				else {
					scope.speedstyle = "color:white";
				}
				var string;
				string = "ACC: " + formatAcceleration(accelTo100);
				if (accelTo100probe == false && accelTo100 < 1000) {
					string = string + " " + formatAcceleration(accelTo200);
				}
				else {
					string = string + " ---";
				}
				
				if (accelTo100probe == false && accelTo100 < 1000 && accelTo200probe == false && accelTo200 < 1000) {
					string = string + " " + formatAcceleration(accelTo300);
				}
				else {
					string = string + " ---";
				}
				
				scope.airspeed = string;
/*				scope.airspeed = "ACC: " +	formatAcceleration(accelTo100) + " " +
											formatAcceleration(accelTo200) + " " +
											formatAcceleration(accelTo300);
*/
			}
			
			//------------------------------------------------------------
			// *** Höhe (Altitude)
			if(streams.electrics.altitude) {
				myAltitude = streams.electrics.altitude;	// Copy alt for global use
				if(altiMode == 1) {
					scope.altimeter = "ALT: " + (streams.electrics.altitude).toFixed(1) + " m";
				}
				else {
					scope.altimeter = "ALT rel: " + (streams.electrics.altitude - myZeroAltitude).toFixed(1) + " m";
				}
			}
			else {
				scope.altimeter = "ALT: !?";
			}

			//------------------------------------------------------------
			// *** Gewicht des Fahrzeuges anzeigen
			scope.vehicleweight = getVehicleWeight(streams, wghtMode);
			if(wghtValid == true) {
				scope.wghtstyle = "color:white";
			}
			else {
				scope.wghtstyle = "color:LightGray";
			}
			
			//------------------------------------------------------------
			// *** Kraftstoffanzeige
			if(streams.electrics.fuel) {						// Fuel Stream vorhanden?
				// Noch mehr als 5% Treibstoff? Oder in der Blink Phase A?
				if (Math.round(streams.electrics.fuel*100) >= 6 || currentDate.getSeconds() % 2 == 0) {
					// Farbe abhandeln (diesmal so eben...)
					if(Math.round(streams.electrics.fuel*100) >= 6) {
						scope.fuelstyle = "color:white";
					}
					else {
						scope.fuelstyle = "color:PeachPuff";
					}

					if (fuelMode == 1) {
						scope.fuel = "FUEL: (" + streams.engineInfo[12].toFixed(0) + ") " + streams.engineInfo[11].toFixed(1) + " l";
					}
					else if (fuelMode == 2) {
//						scope.fuel = "FUEL: " + Math.round(streams.electrics.fuel*100) + "%";	// Fuel percentage
						scope.fuel = "FUEL: " + (streams.electrics.fuel*100).toFixed(1) + "%";	// Fuel percentage
					}
				}
				else {											// Blink Phase B
					scope.fuelstyle = "color:PeachPuff";
					scope.fuel = ">";
				}
			}
			else if(streams.electrics.airfuel) {				// Airfuel Stream vorhanden
				// Noch mehr als 5% Treibstoff?
				if (Math.round(streams.electrics.airfuel) >= 10) {
					scope.fuelstyle = "color:white";
					scope.fuel = "AIRFUEL: " + streams.electrics.airfuel.toFixed(2) + " l";	// Airfuel litre (derived from dg100 glider plane)
				}
				else {	// Reserve!
					if(currentDate.getSeconds() % 2 == 0) {		// Blink Phase A
						scope.fuelstyle = "color:PeachPuff";
						scope.fuel = "AIRFUEL: " + streams.electrics.airfuel.toFixed(2) + " l";	// Airfuel litre (derived from dg100 glider plane)
					}
					else {										// Blink Phase B
						scope.fuelstyle = "color:PeachPuff";
						scope.fuel = ">";
					}
				}
			}
			else {												// Keine Fuel Streams vorhanden?
				scope.fuelstyle = "color:LightGray";
				scope.fuel = "FUEL: -no sensor-";
			}	

			//------------------------------------------------------------
			// *** Anzeige Temperatur

			// Special Case: Oil or Coolant overheat: Automatically select the appropriate mode
			if (streams.electrics.oiltemp && streams.electrics.watertemp) {	// Beide Stream vorhanden?
				if (enableAutoHeatOverride == true) {						// Umschaltung zulassen?
					if (streams.electrics.oiltemp >= oilOverheat
					&& streams.electrics.watertemp < waterOverheat
					&& tempMode != 2)	{		// Oil overheat but not water overheat
						autoHeatOverrideEngaged = true;
						tempMode = 2;
					}
					if (streams.electrics.oiltemp < oilOverheat
					&& streams.electrics.watertemp >= waterOverheat
					&& tempMode != 1)	{	// Water overheat but not oil overheat
						autoHeatOverrideEngaged = true;
						tempMode = 1;
					}
				}
			}

			// Umschaltung (Zickenverhalten) nach einer Zeit wieder zulassen
			if (currentDate.getSeconds() != oldSeconds) {
				oldSeconds = currentDate.getSeconds();					// Tick update
				goatHeatTimer = goatHeatTimer - 1;						// Zicken Timer countdown
			}
			if (goatHeatTimer <= 0) {
				goatHeatTimer = goatHeatTime;							// Timer neu laden
				enableAutoHeatOverride = true;							// Umschaltung wieder zulassen
			}

			// OIL
			if (tempMode == 2) {		// Mode Oil Temp
				if(streams.electrics.oiltemp) {							// Stream vorhanden?
					if(streams.electrics.oiltemp < oilOverheat) {		// Not too hot?
						scope.tempstyle = "color:white";
						scope.temp = "OIL: " + streams.electrics.oiltemp.toFixed(1) + " °C";
					}
					else {												// Too hot
						if(currentDate.getSeconds() % 2 == 0) {			// Blink Phase A?
							scope.tempstyle = "color:PeachPuff";
							scope.temp = "OIL: " + streams.electrics.oiltemp.toFixed(1) + " °C";
						}
						else {											// Blink Phase B?
							scope.tempstyle = "color:PeachPuff";
							scope.temp = ">";
						}
					}
				}
				else {													// Stream nicht vorhanden?
					scope.tempstyle = "color:LightGray";
					scope.temp = "OIL: -no sensor-";
				}
			}
			
			// WATER
			else if (tempMode == 1) {	// Mode Water Temp
				if(streams.electrics.watertemp) {						// Stream vorhanden?
					if(streams.electrics.watertemp < waterOverheat) {	// Not too hot?
						scope.tempstyle = "color:white";
						scope.temp = "WATER: " + streams.electrics.watertemp.toFixed(1) + " °C";
					}
					else {												// Too hot
						if(currentDate.getSeconds() % 2 == 0) {			// Blink Phase A?
							scope.tempstyle = "color:PeachPuff";
							scope.temp = "WATER: " + streams.electrics.watertemp.toFixed(1) + " °C";
						}
						else {											// Blink Phase B?
							scope.tempstyle = "color:PeachPuff";
							scope.temp = ">";
						}
					}
				}
				else {													// Stream nicht vorhanden?
					scope.tempstyle = "color:LightGray";
					scope.temp = "WATER: -no sensor-";
				}
			}

			dt = performance.now();			
		})	
	});

    scope.clockClick = function () {
		// Modus der Uhrzeit umschalten
		clockMode < 3 ? clockMode += 1 : clockMode = 1
	}
	scope.clockRclick = function () {
		if(clockMode == 3) {
			// Fahrtzeit nullen
			driveTime = 0;
		}
	}

    scope.speedClick = function () {
		// Modus der Geschwindigkeitsanzeige umschalten
		speedMode < 3 ? speedMode += 1 : speedMode = 1
    }
	scope.speedRclick = function () {
		// Wenn im vMax Mode, dann reseten
		if (speedMode == 2) {
			vMax = 0;
		}
		if (speedMode == 3) {
			accelTo100 = 1000;					// (entspricht "nie oder nicht getestet")
			accelTo200 = 1000;
			accelTo300 = 1000;
		}

	}

    scope.altClick = function () {
		// Modus des Höhenmessers umschalten
		altiMode < 2 ? altiMode += 1 : altiMode = 1
	}
	scope.altRclick = function () {
		if(altiMode == 2) {
			// Relativen Höhenmesser nullen
			myZeroAltitude = myAltitude;
		}
	}

    scope.wghtClick = function () {
		// Modus der Gewichtsmessung umschalten
		wghtMode < 3 ? wghtMode += 1 : wghtMode = 1
	}
	scope.wghtRclick = function () {
		if(wghtMode == 3) {
			// Dummy
			wghtMode = 3;
		}
	}

    scope.fuelClick = function () {
		// Modus der Tankanzeige umschalten
		fuelMode < 2 ? fuelMode += 1 : fuelMode = 1
     }

    scope.tempClick = function () {
		// Modus der Temperaturanzeige umschalten
		if (autoHeatOverrideEngaged == true) {
			autoHeatOverrideEngaged = false;
			enableAutoHeatOverride = false;
			goatHeatTimer = goatHeatTime;							// Timer neu laden
		}
		tempMode < 2 ? tempMode += 1 : tempMode = 1
     }


	// -------------------------------------------------------------------------------
	// Make sure we clean up after closing the app.
	scope.$on('$destroy', function () {
		StreamsManager.remove(streamsList);

		// Save data of interest for the next session
		var data = {}	// Container

		data.clockMode = clockMode;
		data.driveTime = driveTime;
		data.speedMode = speedMode;
		data.vMax = vMax;
		data.accelTo100 = accelTo100;
		data.accelTo200 = accelTo200;
		data.accelTo300 = accelTo300;
		data.altiMode = altiMode;
		data.myZeroAltitude = myZeroAltitude;
		data.fuelMode = fuelMode;
		data.tempMode = tempMode;
		data.wghtMode = wghtMode;

		data = JSON.stringify(data);
		bngApi.engineLua("writeFile('" + settings_file_path + "'," + "'" + data + "', true)");
	});	
},
};
}]);